﻿// DualModeFolderEditorForm.cs
//
// An editing Form for the custom server control HyperlinkFileList
//
// This Form is launched from the DualModeFolderEditor class
//
// Author:  Steve Wellens
// Company: EndWell Software, Inc.
// Copyright (c) 2009 EndWell Software, Inc.
// ---------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Web.UI.Design;

namespace EndWell
{
    public partial class DuaModeFolderEditorForm : Form
    {
        // virtual dirs start with this
        const String VIRT_DIR_START = "~/";  

        public String m_Value;   // read from outside

        private ITypeDescriptorContext m_Context;
        private IWebApplication m_WebApplication;
        private IProjectItem m_RootFileProjectItem;

        // ---- Constructor --------------------

        public DuaModeFolderEditorForm(String Input, ITypeDescriptorContext Context)
        {
            InitializeComponent();
 
            m_Value = FixSlashes(Input);
            m_Context = Context;
        }

        // ---- Form Load --------------------------------
        // collect some info from Visual Studio

        private void DuaModeFolderEditorForm_Load(object sender, EventArgs e)
        {
            HyperlinkFileList Control = m_Context.Instance as HyperlinkFileList;

            m_WebApplication = Control.Page.Site.GetService(typeof(IWebApplication)) as IWebApplication;

            if (m_WebApplication != null)
            {
                // get the Root project Item
                m_RootFileProjectItem = m_WebApplication.GetProjectItemFromUrl(VIRT_DIR_START);
            }
            else
            {
                labelMessage.Text = "Error: Couldn't get Web Application";
            }
            
            textBoxFolder.Text = m_Value;

            // guess as to which radiobutton should be checked
            if (m_Value.StartsWith(VIRT_DIR_START))
                radioButtonVirtual.Checked = true;
            else
                radioButtonAbsolute.Checked = true;
        }
    
        // ---- OK Button ---------------------------------

        private void buttonOK_Click(object sender, EventArgs e)
        {
            m_Value = textBoxFolder.Text;
        }

        // ---- Browse Button ------------------------------

        private void buttonBrowse_Click(object sender, EventArgs e)
        {
            // warning: This can crash Visual Studio
             System.Diagnostics.Debugger.Break();

            labelMessage.Text = "";

            FolderBrowserDialog Dlg = new FolderBrowserDialog();
            Dlg.Description = "Select the folder of the files to display";

            // try to start the browser in the correct directory
            // if path is passed, try to use it
            if (String.IsNullOrEmpty(textBoxFolder.Text) == false)
            {
                Dlg.RootFolder = Environment.SpecialFolder.MyComputer;

                String DirPath = textBoxFolder.Text;
                if (DirPath.StartsWith(VIRT_DIR_START) == true)
                {
                    // try to get physical path from virtual path
                    IProjectItem ProjectItem = m_WebApplication.GetProjectItemFromUrl(DirPath);
                    if (ProjectItem != null)
                        Dlg.SelectedPath = ProjectItem.PhysicalPath;
                }
                else   // physical Path
                {                   
                    // Doesn't work unless back slashes are used...sigh.
                    Dlg.SelectedPath = DirPath.Replace("/", @"\");
                }
            }
            else if (radioButtonAbsolute.Checked == true)
                Dlg.RootFolder = Environment.SpecialFolder.MyComputer;
            else
                Dlg.SelectedPath = m_RootFileProjectItem.PhysicalPath;

            // Launch the dialog box
            if (Dlg.ShowDialog() == DialogResult.OK)
            {
                Dlg.SelectedPath = FixSlashes(Dlg.SelectedPath);

                // we always get an absolute path back, if virtual
                // radio button is selected, try and convert it.
                if (radioButtonVirtual.Checked == true)
                {
                    textBoxFolder.Text = AbsoluteToVirtual(Dlg.SelectedPath);                    
                }
                else
                {
                    textBoxFolder.Text = Dlg.SelectedPath; 
                }
            }
        }

        // ---- Radio Button clicks ------------------
        //
        // If there is an existing path, try to convert it

        private void radioButtonAbsolute_Click(object sender, EventArgs e)
        {
            labelMessage.Text = "";
            textBoxFolder.Text = VirtualToAbsolute(textBoxFolder.Text);
        }

        private void radioButtonVirtual_Click(object sender, EventArgs e)
        {
            labelMessage.Text = "";
            textBoxFolder.Text = AbsoluteToVirtual(textBoxFolder.Text);
        }

        // ---- AbsoluteToVirtual ----------------------
        //
        // Convert path like D:\websites\MyWebsite\xml
        // To: ~/xml

        private String AbsoluteToVirtual(String Absolute)
        {
            if (String.IsNullOrEmpty(Absolute))
                return Absolute;

            if (Absolute.StartsWith(VIRT_DIR_START) == true)
            {
                // already in virtual form
                return Absolute;
            }

            String PhysicalRootPath = FixSlashes(m_RootFileProjectItem.PhysicalPath);

            if (Absolute.StartsWith(PhysicalRootPath) == true)
            {
                String Temp = Absolute.Replace(PhysicalRootPath, VIRT_DIR_START);
                return FixSlashes(Temp);
            }
            else
            {
                labelMessage.Text = "Couldn't resolve absolute path to virtual path";
                return Absolute;
            }
        }

        // ---- VirtualToAbsolute -------------------------------------
        //
        // Convert path like ~/xml
        // To: D:\websites\MyWebsite\xml

        private String VirtualToAbsolute(String Virtual)
        {
            if (String.IsNullOrEmpty(Virtual))
                return Virtual;

            if (Virtual.StartsWith(VIRT_DIR_START) == true)
            {
                if (m_WebApplication != null)
                {
                    IProjectItem ProjectItem = m_WebApplication.GetProjectItemFromUrl(Virtual);
                    if (ProjectItem != null)
                    {
                        String Temp = ProjectItem.PhysicalPath;
                        return FixSlashes(Temp);
                    }
                }
            }
           // something didn't work, return original path
           return Virtual;
        }

        // ---- FixSlashes ----------------------
        //
        // The GetProjectItemFromUrl function wants forward slashes.
        // For consistency, use foward slashes everywhere.
        // For consistency, always have a trailing slash.

        private String FixSlashes(String Input)
        {
            if (Input == "")
                return Input;

            String Temp = Input.Replace(@"\", "/");

            // add trailing slash if there isn't one.
            if (Temp.EndsWith("/") == false)
                Temp = Temp + "/";
            return Temp;
        }
    }
}

